import 'dart:typed_data';

import 'package:camera/camera.dart';
import 'package:flutter/services.dart';

class CameraImageConverter {
  static MethodChannel _platform =
      const MethodChannel('camera_image_converter');

  ///  Transform given image to JPEG compressed through native code.
  ///
  ///  Function gets [CameraImage] in YUV format for processing and returns
  ///  [Uint8List] of JPEG bytes.
  /// [quality] is a value between 0 and 100 representing the compression level
  static Future<Uint8List> convertCameraImageTo(CameraImage image,
      {int quality = 100}) async {
    assert(quality > 0);
    assert(quality <= 100);

    List<int> strides = new Int32List(image.planes.length * 2);
    int index = 0;
    // We need to transform the image to Uint8List so that the native code could
    // transform it to byte[]
    List<Uint8List> data = image.planes.map((plane) {
      strides[index] = (plane.bytesPerRow);
      index++;
      strides[index] = (plane.bytesPerPixel);
      index++;
      return plane.bytes;
    }).toList();
    Uint8List jpegImage = await _platform.invokeMethod('convertCameraImage', {
      'platforms': data,
      'height': image.height,
      'width': image.width,
      'strides': strides,
      'quality': quality,
    });

    return jpegImage;
  }
}
